# Generated by Django 4.0.5 on 2022-07-16 06:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('publicurls', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='publicurl',
            options={'ordering': ['-created'], 'verbose_name': 'Public URL', 'verbose_name_plural': 'Public URLs'},
        ),
        migrations.AddField(
            model_name='publicurl',
            name='thankyou_text',
            field=models.TextField(default='Thank you', help_text='Will be shown if a form has successfully been submitted.', verbose_name='Thank-you text'),
        ),
    ]
