# Generated by Django 4.0.5 on 2022-07-24 12:58

from django.db import migrations, models

import basxbread.contrib.publicurls.models


class Migration(migrations.Migration):

    dependencies = [
        ('publicurls', '0002_alter_publicurl_options_publicurl_thankyou_text'),
    ]

    operations = [
        migrations.AlterField(
            model_name='publicurl',
            name='url',
            field=models.CharField(help_text='Internal URL', max_length=2048, validators=[basxbread.contrib.publicurls.models.validate_url], verbose_name='URL'),
        ),
    ]
