# Generated by Django 4.0.5 on 2022-08-15 09:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_templates', '0004_documenttemplatevariable_template'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttemplate',
            name='filename_template',
            field=models.TextField(blank=True, verbose_name='Filename template'),
        ),
    ]
