//
// Copyright IBM Corp. 2016, 2018
//
// This source code is licensed under the Apache-2.0 license found in the
// LICENSE file in the root directory of this source tree.
//

//-----------
// v10 COLORS
//-----------
@import './vendor/@carbon/elements/scss/colors/colors';
